c----------------------------------------------------------------------
      program surfcsv
c----------------------------------------------------------------------
c
c --- SURFCSV   Version: 7.0.0           Level: 141107           MAIN
c               D. Strimaitis
c 
c     Copyright (c) 2011-2014 by Exponent, Inc.
c
c --- PURPOSE:  Read CSV-format met data file, extract data at
c               fixed intervals, and output as SURF.DAT v2.1
c               with begin/end times that span the interval.
c
c --- UPDATES:
c
c --- TNG-1.0.1 (141106) to v7.0.0 (141107)
c     1. Update to Version 7 System
c        Control file is dataset version 7.0
c        Modified: RDCF
c
c --- V1.0 (110118) to TNG-1.0.1 (141106)
c     1. Correct the screen used for identifying missing wind speeds
c        Temperature was used instead of wind speed when checking for
c        speeds < 0 ( which readily occurs when temperature data are
c        in degrees C) causing valid wind speeds to be replaced by
c        missing value.  Missing speeds were not properly trapped.
c        --- Bug-fix reported by Tom Wickstrom
c        Modified: RDCSV
c
c --- SURFCSV calls:  RDCF, QAYR4, YR4, JULDAY, WRHD, WRS, YR4, DEDAT,
c                     RDCSV
c----------------------------------------------------------------------
c --- Set up parameters
      parameter (io6=7, iinp=8, iout=10, ictrl=11, mxkeep=2)

c --- Include Files
      include 'ctrl.scs'

      character*4 cname
      character*12 cver,clev
      character*16 clat,clon

      integer iceil(mxkeep),icc(mxkeep),irh(mxkeep),ipcode(mxkeep),
     &        ibuf(3,mxkeep),mdhrsec(mxkeep)

      real ws(mxkeep),wd(mxkeep),tempk(mxkeep),pres(mxkeep)

      character*80 filnam1, filnam2, ctrlnam
      character*16 dataset

      logical lproblem

c --- Identify processor
      data cver/'7.0.0       '/
      data clev/'141107      '/

c --- Set the SURF.DAT file type to formatted, without station
c --- location information, for 1 station
      data iform/2/, ienvpop/0/, nsta/1/
      data anem/10.0/, cname/'CSV '/
      data clat/'0.0N            '/,clon/'0.0E            '/

c --- Set missing values
      data rmiss/9999./, imiss/9999/

c --- Set file names
      data filnam1/'csv.dat'/
      data filnam2/'surf.dat'/
      data ctrlnam/'surfcsv.inp'/

c --- Assign missings
      wsx=rmiss
      wdx=rmiss
      iceilx=imiss
      iccx=imiss
      tempkx=rmiss
      irhx=imiss
      presx=rmiss
      ipcodex=imiss

c --- Open the files
      open(io6,file='surfcsv.lst')
      open(ictrl,file=ctrlnam,status='old')
      open(iinp,file=filnam1,status='old')
      open(iout,file=filnam2,status='unknown')

c --- Read control file
      call RDCF(ictrl)

c --- Set century using starting year
      call QAYR4(io6,ibyr,0,ierr)
      if(ierr.NE.0) stop 'Use a 4-digit year for start of run'

c --- Make the year conform to YYYY format
      call YR4(io6,ibyr,ierr)
      if(ierr.NE.0) stop 'Halted in MAIN - see list file'
      call YR4(io6,ieyr,ierr)
      if(ierr.NE.0) stop 'Halted in MAIN - see list file'
c --- Get Julian day
      call JULDAY(io6,ibyr,ibmo,ibdy,ibjul)
      call JULDAY(io6,ieyr,iemo,iedy,iejul)

c --- Write header for SURF.DAT
      call WRHD(ienvpop,io6,iform,iout,ibyr,ibjul,ibhr,ibsec,
     1           ieyr,iejul,iehr,iesec,atzone,nsta,ipack,idsta,
     2           anem,cname,clat,clon,cver,clev)

c --- Make date-hr variables for full range of data
      ibdathr = ibyr * 100000 + ibjul * 100 + ibhr
      iedathr = ieyr * 100000 + iejul * 100 + iehr
c --- Make date-hr-sec variables relative to starting year
c --- YJJJHHSSSS where Y is 0 or 1, satisfies integer*4 limit
      iydiff=ieyr-ibyr
      if(iydiff.GT.1 .OR. iydiff.LT.0) then
         write(io6,*)'Date range must be < 2 years and > 0'
         write(io6,*)'Starting date: ',ibyr,ibmo,ibdy,ibsec
         write(io6,*)'  Ending date: ',ieyr,iemo,iedy,iesec
         stop 'Halted in MAIN - see list file'
      endif
      ibhrsec=ibjul*1000000+ibhr*10000+ibsec
      iehrsec=iydiff*1000000000+iejul*1000000+iehr*10000+iesec

c --- Set window first period
c --- Start-time of first window for this output time
      iyr0=ibyr
      ijul0=ibjul
      ihr0=ibhr
      isec0=ibsec
c --- End-time of first window for this output time
      iyr=iyr0
      ijul=ijul0
      ihr=ihr0
      isec=isec0
      call INCRS(io6,iyr,ijul,ihr,isec,itsec)
c --- Set date-hr-sec variable to end of first window
      iydiff=iyr-ibyr
      idhrsec=iydiff*1000000000+ijul*1000000+ihr*10000+isec

c --- Process data records
c ------------------------

c --- Skip any header records
      do i=1,nskip
         read(iinp,*)
      enddo

c --- Extract first data record from file
      call RDCSV(iinp,io6,myr,mjul,mhr,msec,imiss,rmiss,
     &           ws(1),wd(1),iceil(1),icc(1),tempk(1),
     &           irh(1),pres(1),ipcode(1),idone)
c --- Make the year conform to YYYY format
      call YR4(io6,myr,ierr)
      if(ierr.NE.0) stop 'Halted in MAIN - see list file'
      mdathr = myr * 100000 + mjul * 100 + mhr
      mydiff=myr-ibyr
      mdhrsec(1)=mydiff*1000000000+mjul*1000000+mhr*10000+msec

c --- Extract second data record from file
      call RDCSV(iinp,io6,myr,mjul,mhr,msec,imiss,rmiss,
     &           ws(2),wd(2),iceil(2),icc(2),tempk(2),
     &           irh(2),pres(2),ipcode(2),idone)
c --- Make the year conform to YYYY format
      call YR4(io6,myr,ierr)
      if(ierr.NE.0) stop 'Halted in MAIN - see list file'
      mdathr = myr * 100000 + mjul * 100 + mhr
      mydiff=myr-ibyr
      mdhrsec(2)=mydiff*1000000000+mjul*1000000+mhr*10000+msec

10    continue
      if(idone.EQ.1) then
c ---    No more data can be read from file
c ---    Write SURF.DAT record using missings
         call WRS(iform,nsta,ipack,iout,io6,ibuf,
     &            iyr0,ijul0,ihr0,isec0,iyr,ijul,ihr,isec,
     &            wsx,wdx,iceilx,iccx,tempkx,irhx,presx,ipcodex)
         itnew=1
         idnew=0
      elseif(idhrsec.EQ.mdhrsec(1)) then
c ---    Data timestamp(1) matches target time for output
c ---    Write SURF.DAT record using data(1)
         call WRS(iform,nsta,ipack,iout,io6,ibuf,
     &            iyr0,ijul0,ihr0,isec0,iyr,ijul,ihr,isec,
     &            ws,wd,iceil,icc,tempk,irh,pres,ipcode)
         itnew=1
         idnew=1
      elseif(idhrsec.LT.mdhrsec(1)) then
c ---    Target is before current data range
c ---    Write SURF.DAT record using missings
         call WRS(iform,nsta,ipack,iout,io6,ibuf,
     &            iyr0,ijul0,ihr0,isec0,iyr,ijul,ihr,isec,
     &            wsx,wdx,iceilx,iccx,tempkx,irhx,presx,ipcodex)
         itnew=1
         idnew=0
      elseif(idhrsec.GE.mdhrsec(2)) then
c ---    Target is after current data range
         itnew=0
         idnew=1
      else
c ---    Target is between endpoints of current data range
         if(mtime.EQ.1) then
c ---       No exact match, so write SURF.DAT record using missings
            call WRS(iform,nsta,ipack,iout,io6,ibuf,
     &               iyr0,ijul0,ihr0,isec0,iyr,ijul,ihr,isec,
     &               wsx,wdx,iceilx,iccx,tempkx,irhx,presx,ipcodex)
            itnew=1
            idnew=0
         else
c ---       Pick nearest in time
            i1=idhrsec-mdhrsec(1)
            i2=mdhrsec(2)-idhrsec
            if(i1.LE.i2) then
c ---          Write SURF.DAT record using data(1)
               call WRS(iform,nsta,ipack,iout,io6,ibuf,
     &                  iyr0,ijul0,ihr0,isec0,iyr,ijul,ihr,isec,
     &                  ws,wd,iceil,icc,tempk,irh,pres,ipcode)
            else
c ---          Write SURF.DAT record using data(2)
               call WRS(iform,nsta,ipack,iout,io6,ibuf,
     &                  iyr0,ijul0,ihr0,isec0,iyr,ijul,ihr,isec,
     &                  ws(2),wd(2),iceil(2),icc(2),tempk(2),
     &                  irh(2),pres(2),ipcode(2))
            endif
            itnew=1
            idnew=0
         endif
      endif

c --- Increment time if record was written
      if(itnew.EQ.1) then
         iyr0=iyr
         ijul0=ijul
         ihr0=ihr
         isec0=isec
         call INCRS(io6,iyr,ijul,ihr,isec,itsec)
         idathr = iyr * 100000 + ijul * 100 + ihr
         iydiff=iyr-ibyr
         idhrsec=iydiff*1000000000+ijul*1000000+ihr*10000+isec
         if(idhrsec.GT.iehrsec) goto 999
      endif

c --- Get more data when needed
      if(idnew.EQ.1) then
c ---    Copy data(2) data into data(1) prior to next input
         mdhrsec(1)=mdhrsec(2)
         ws(1)=ws(2)
         wd(1)=wd(2)
         iceil(1)=iceil(2)
         icc(1)=icc(2)
         tempk(1)=tempk(2)
         irh(1)=irh(2)
         pres(1)=pres(2)
         ipcode(1)=ipcode(2)
c ---    Update data(2) if available
         if(idhrsec.GT.mdhrsec(1) .AND. mdhrsec(2).LT.iehrsec) then
            call RDCSV(iinp,io6,myr,mjul,mhr,msec,imiss,rmiss,
     &                 ws(2),wd(2),iceil(2),icc(2),tempk(2),
     &                 irh(2),pres(2),ipcode(2),idone)
            call YR4(io6,myr,ierr)
            if(ierr.NE.0) stop 'Halted in MAIN - see list file'
            mdathr = myr * 100000 + mjul * 100 + mhr
            mydiff=myr-ibyr
            mdhrsec(2)=mydiff*1000000000+mjul*1000000+mhr*10000+msec
         endif
      endif

c --- Process period
      goto 10

999   stop
      end

c----------------------------------------------------------------------
      subroutine rdcf(io)
c----------------------------------------------------------------------
c
c --- SURFCSV  Version: 7.0.0         Level: 141107               RDCF
c ---          D. Strimaitis
c
c --- PURPOSE:  Reads control file for SURFCSV application
c
c --- UPDATES:
c --- v1.0 (110118) to v7.0.0 (141107)
c     1. Control file is dataset version 1.0 or 7.0
c
c --- INPUTS:
c
c --- OUTPUT:
c                 ibyr,ibmo,ibdy,ibhr,ibsec,ieyr,iemo,iedy,iehr,iesec,
c                 atzone,idsta,itsec,mtime,nskip,icdate,ictime,nmvars,
c                 icws,icwd,ictemp,icrh,icpress,icceil,iccld,icsky,
c                 ifdatime,fws,mwd,mtemp,frh,fpress,fceil,fcld
c
c --- RDCF called by: (main)
c --- RDCF calls:     CASE320
c----------------------------------------------------------------------
      include 'ctrl.scs'

c --- Local variables
      logical lerror
      character*1 sh
      character*8 units
      character*16 dataset,dataver,datetime
      character*180 char180,blnk180
      character*320 char320,blnk320

      data sh/'#'/
      data lerror/.FALSE./

      do i=1,320
         blnk320(i:i)=' '
      enddo
      blnk180=blnk320(1:180)

c --- Set initial values
      icdate=0
      ictime=0
      icws=0
      icwd=0
      ictemp=0
      icrh=0
      icpress=0
      icceil=0
      iccld=0
      icsky=0
      atzone='UTC+hhmm'
      idsta=0

c --- Process first line of control file
      read(io,'(2a16)') dataset,dataver
      if(dataset.NE.'SURFCSV.INP     ') then
         write(*,*)'Error in control file '
         write(*,*)'Expected DATASET name: SURFCSV.INP'
         write(*,*)'Found    DATASET name: ',dataset
         stop 'HALTED in RDCF'
      endif
      if(dataver.NE.'1.0             ' .AND.
     &   dataver.NE.'7.0             ') then
         write(*,*)'Error in control file'
         write(*,*)'Expected VERSION: 1.0 or 7.0'
         write(*,*)'Found    VERSION: ',dataver
         stop 'HALTED in RDCF'
      endif

c --- Number of comment records
      read(io,*) ncom

c --- Skip comments
      do n=1,ncom
         read(io,*)
      enddo

c --- Initialize number of variables
      nmvars=0

c --- Read each data record and extract information before comment
20    char180=blnk180
      units='        '
      datetime='                '
      read(io,'(a180)',end=21) char180

c --- Create upper-case version of line
      char320=blnk320
      char320(1:180)=char180
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)

c --- Position of comment delimiter
      ic=INDEX(char320,sh)
      if(ic.EQ.0) then
c ---    No delimiter, skip this line
         goto 20
      else
         i0=1
         i1=ic-1
      endif

c --- Go through list of variables
c --------------------------------

c --- ID
      if(INDEX(char320,'STATION_ID').NE.0) then
         read(char320(i0:i1),'(i)') idsta

c --- Time Zone
      elseif(INDEX(char320,'TIME_ZONE').NE.0) then
         atzone=char320(i0:i1)
         atzone=ADJUSTL(atzone)

c --- Output Starting_Time
      elseif(INDEX(char320,'STARTING_TIME').NE.0) then
         read(char320(i0:i1),*) ibyr,ibmo,ibdy,ibhr,ibsec
c ---    Convert minutes to seconds
         ibsec=60*ibsec

c --- Output Ending_Time
      elseif(INDEX(char320,'ENDING_TIME').NE.0) then
         read(char320(i0:i1),*) ieyr,iemo,iedy,iehr,iesec
c ---    Convert minutes to seconds
         iesec=60*iesec

c --- Output Time_Increment
      elseif(INDEX(char320,'TIME_INCREMENT').NE.0) then
         read(char320(i0:i1),*) ithh,itsec
c ---    Convert increment to seconds
         itsec=60*itsec+3600*ithh

c --- Output Time_Method
      elseif(INDEX(char320,'TIME_METHOD').NE.0) then
         read(char320(i0:i1),*) mtime

c --- Number of header records
      elseif(INDEX(char320,'SKIP_HEADERS').NE.0) then
         read(char320(i0:i1),*) nskip

c --- DATE_column
      elseif(INDEX(char320,'DATE_COLUMN').NE.0) then
         read(char320(i0:i1),*) icdate
         nmvars=MAX(nmvars,icdate)

c --- TIME_column (may be same as DATE_COLUMN)
      elseif(INDEX(char320,'TIME_COLUMN').NE.0) then
         read(char320(i0:i1),*) ictime
         nmvars=MAX(nmvars,ictime)

c --- WINDSPEED_column
      elseif(INDEX(char320,'WINDSPEED_COLUMN').NE.0) then
         read(char320(i0:i1),*) icws
         nmvars=MAX(nmvars,icws)

c --- WINDDIRECTION_column
      elseif(INDEX(char320,'WINDDIRECTION_COLUMN').NE.0) then
         read(char320(i0:i1),*) icwd
         nmvars=MAX(nmvars,icwd)

c --- TEMPERATURE_column
      elseif(INDEX(char320,'TEMPERATURE_COLUMN').NE.0) then
         read(char320(i0:i1),*) ictemp
         nmvars=MAX(nmvars,ictemp)

c --- RELHUMIDITY_column
      elseif(INDEX(char320,'RELHUMIDITY_COLUMN').NE.0) then
         read(char320(i0:i1),*) icrh
         nmvars=MAX(nmvars,icrh)

c --- PRESSURE_column
      elseif(INDEX(char320,'PRESSURE_COLUMN').NE.0) then
         read(char320(i0:i1),*) icpress
         nmvars=MAX(nmvars,icpress)

c --  CLOUD_column
      elseif(INDEX(char320,'CLOUD_COLUMN').NE.0) then
         read(char320(i0:i1),*) icsky
         nmvars=MAX(nmvars,icsky)

c --- CEILING_column
      elseif(INDEX(char320,'CEILING_COLUMN').NE.0) then
         read(char320(i0:i1),*) icceil
         nmvars=MAX(nmvars,icceil)

c --- CLOUDCOVER_column
      elseif(INDEX(char320,'CLOUDCOVER_COLUMN').NE.0) then
         read(char320(i0:i1),*) iccld
         nmvars=MAX(nmvars,iccld)

c --- DATE and TIME Format (applies to either combined or
c --- single columns)
      elseif(INDEX(char320,'DATETIME_FORMAT').NE.0) then
         datetime=char320(i0:i1)
         datetime=ADJUSTL(datetime)
         if(datetime.EQ.'YYYY-MM-DD HH:MM') then
            ifdatime=1
         elseif(datetime.EQ.'MM/DD/YYYY HH:MM') then
            ifdatime=2
         else
            write(*,*)'Error in control file for application'
            write(*,*)'Invalid DATETIME_FORMAT'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- WINDSPEED_units
      elseif(INDEX(char320,'WINDSPEED_UNITS').NE.0) then
         read(char320(i0:i1),*) units
         units=ADJUSTL(units)
         if(units.EQ.'MPS     ') then
            fws=1.0
         elseif(units.EQ.'MPH     ') then
            fws=0.44704
         elseif(units.EQ.'KTS     ') then
            fws=0.51444
         else
            write(*,*)'Error in control file for application'
            write(*,*)'Invalid WINDSPEED_UNITS'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- WINDDIRECTION_units
      elseif(INDEX(char320,'WINDDIRECTION_UNITS').NE.0) then
         read(char320(i0:i1),*) units
         units=ADJUSTL(units)
         if(units.EQ.'DEG     ') then
            mwd=1
         elseif(units.EQ.'NSEW    ') then
            mwd=0
         else
            write(*,*)'Error in control file for application'
            write(*,*)'Invalid WINDDIRECTION_UNITS'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- TEMPERATURE_units
      elseif(INDEX(char320,'TEMPERATURE_UNITS').NE.0) then
         read(char320(i0:i1),*) units
         units=ADJUSTL(units)
         if(units.EQ.'K       ') then
            mtemp=1
         elseif(units.EQ.'C       ') then
            mtemp=2
         elseif(units.EQ.'F       ') then
            mtemp=3
         else
            write(*,*)'Error in control file for application'
            write(*,*)'Invalid TEMPERATURE_UNITS'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- RELHUMIDITY_units
      elseif(INDEX(char320,'RELHUMIDITY_UNITS').NE.0) then
         read(char320(i0:i1),*) units
         units=ADJUSTL(units)
         if(units.EQ.'%       ') then
            frh=1.0
         elseif(units.EQ.'FRAC    ') then
            frh=100.0
         else
            write(*,*)'Error in control file for application'
            write(*,*)'Invalid RELHUMIDITY_UNITS'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- PRESSURE_units
      elseif(INDEX(char320,'PRESSURE_UNITS').NE.0) then
         read(char320(i0:i1),*) units
         units=ADJUSTL(units)
         if(units.EQ.'MB      ') then
            fpress=1.0
         elseif(units.EQ.'IN      ') then
            fpress=33.7685
         elseif(units.EQ.'ATM     ') then
            fpress=1013.25
         else
            write(*,*)'Error in control file for application'
            write(*,*)'Invalid PRESSURE_UNITS'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- CEILING_units
      elseif(INDEX(char320,'CEILING_UNITS').NE.0) then
         read(char320(i0:i1),*) units
         units=ADJUSTL(units)
         if(units.EQ.'100FT   ') then
            fceil=1.0
         elseif(units.EQ.'FT      ') then
            fceil=0.01
         elseif(units.EQ.'M       ') then
            fceil=0.0328084
         elseif(units.EQ.'KM      ') then
            fceil=32.8084
         else
            write(*,*)'Error in control file for application'
            write(*,*)'Invalid CEILING_UNITS'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- CLOUDCOVER_units
      elseif(INDEX(char320,'CLOUDCOVER_UNITS').NE.0) then
         read(char320(i0:i1),*) units
         units=ADJUSTL(units)
         if(units.EQ.'TENTHS  ') then
            fcld=1.0
         else
            write(*,*)'Error in control file for application'
            write(*,*)'Invalid CLOUDCOVER_UNITS'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- Found unexpected variable assignment
      else
         write(*,*)'Error in control file for application'
         write(*,*)'Unexpected variable assignment found:'
         write(*,*)char320(1:klast)
         stop 'HALTED in RDCF'
      endif

      goto 20

21    continue

c --- Correct for multiple cloud assignments (keep CLOUD_column)
      isum=iccld+icceil
      if(icsky.GT.0 .AND. isum.GT.0) then
         iccld=0
         icceil=0
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine rdcsv(io,ilst,iyr,juldy,ihh,isec,imiss,rmiss,
     &           ws,wd,iceil,icc,tempk,irh,pres,ipcode,idone)
c----------------------------------------------------------------------
c
c --- SURFCSV  Version: 7.0.0         Level: 141106              RDCSV
c ---          D. Strimaitis
c
c --- PURPOSE:  Reads one data record from CSV file
c
c --- UPDATES:
c --- V1.0 (110118) to TNG-1.0.1 (141106)
c     1. Correct the screen used for identifying missing wind speeds
c        Temperature was used instead of wind speed when checking for
c        speeds < 0 ( which readily occurs when temperature data are
c        in degrees C) causing valid wind speeds to be replaced by
c        missing value.  Missing speeds were not properly trapped.
c        --- Bug-fix reported by Tom Wickstrom
c      
c
c --- INPUTS:
c                 io,ilst,imiss,rmiss
c
c --- OUTPUT:
c                 iyr,juldy,ihh,isec,
c                 ws,wd,iceil,icc,tempk,irh,pres,ipcode,idone
c
c --- RDCSV called by: (main)
c --- RDCSV calls:     COLUMN320, IRANGE, JULDAY
c----------------------------------------------------------------------
      include 'ctrl.scs'

c --- Local variables
      logical lerror
      character*1 sl,ds,cl,cm
      character*8 acld
      character*16 datm
      character*180 char180,blnk180
      character*320 char320,blnk320
      integer icommas(0:99)

      data sl/'/'/
      data ds/'-'/
      data cl/':'/
      data cm/','/
      data lerror/.FALSE./

      do i=1,320
         blnk320(i:i)=' '
      enddo
      blnk180=blnk320(1:180)

c --- Set initial values to missing
      ws=rmiss
      wd=rmiss
      iceil=imiss
      icc=imiss
      tempk=rmiss
      irh=imiss
      pres=rmiss
      ipcode=imiss

c --- Read data record and extract information before comment
20    char320=blnk320
      read(io,'(a320)',end=21) char320

c --- Test line for at least 1 comma to screen out empty lines
      itest=INDEX(char320,cm)
      if(itest.EQ.0) goto 20

c --- Identify column ranges
      kmlast=LEN_TRIM(char320)
      call COLUMN320(char320,kmlast,cm,nmvars,ncommas,icommas)

c --- Date
      if(icdate.GT.0) then
         datm='                '
         call IRANGE(icdate,icommas,ncommas,i1,i2)
         datm=char320(i1:i2)
         if(ifdatime.EQ.1) then
c ---       YYYY-MM-DD  (ds is - )
            m1=INDEX(datm,ds)
            m2=INDEX(datm,ds,back=.TRUE.)
            m3=INDEX(datm,' ')
            mlast=LEN_TRIM(datm)
c ---       Year
            n1=i1
            n2=m1-1
            read(datm(n1:n2),'(i)')iyr
c ---       Month
            n1=m1+1
            n2=m2-1
            read(datm(n1:n2),'(i)')imo
c ---       Day
            n1=m2+1
            if(m3.GT.0) then
               n2=m3-1
            else
               n2=mlast
            endif
            read(datm(n1:n2),'(i)')iday
         else
c ---       MM/DD/YYYY  (sl is /)
            m1=INDEX(datm,sl)
            m2=INDEX(datm,sl,back=.TRUE.)
            m3=INDEX(datm,' ')
            mlast=LEN_TRIM(datm)
c ---       Month
            n1=i1
            n2=m1-1
            read(datm(n1:n2),'(i)')imo
c ---       Day
            n1=m1+1
            n2=m2-1
            read(datm(n1:n2),'(i)')iday
c ---       Year
            n1=m2+1
            if(m3.GT.0) then
               n2=m3-1
            else
               n2=mlast
            endif
            read(datm(n1:n2),'(i)')iyr
         endif
c ---    Compute Julian Day
         call JULDAY(ilst,iyr,imo,iday,juldy)
      endif

c --- Time
      if(ictime.GT.0) then
         datm='                '
         call IRANGE(ictime,icommas,ncommas,i1,i2)
         datm=char320(i1:i2)
c ---    HH:MM  (cl is :)
         m=INDEX(datm,cl)
c ---    Hour
         n1=m-2
         n2=m-1
         read(datm(n1:n2),'(i)')ihh
c ---    Minute
         n1=m+1
         n2=m+2
         read(datm(n1:n2),'(i)')imm
c ---    Convert Minutes to Seconds
         isec=imm*60
      endif

c --- Now data: Screen for negatives as missings
c ----------------------------------------------

c --- Wind Speed
      if(icws.GT.0) then
         call IRANGE(icws,icommas,ncommas,i1,i2)
         call RGET320(char320,i1,i2,wstest)
         if(wstest.GE.0.00) ws=wstest*fws
      endif

c --- Wind Direction
      if(icwd.GT.0) then
         call IRANGE(icwd,icommas,ncommas,i1,i2)
         if(mwd.EQ.1) then
c ---       Number
            call RGET320(char320,i1,i2,wdtest)
            if(wdtest.GE.0.00) wd=wdtest
         else
c ---       String
         endif
      endif

c --- Temperature
      if(ictemp.GT.0) then
         call IRANGE(ictemp,icommas,ncommas,i1,i2)
         call RGET320(char320,i1,i2,ttest)
         if(ttest.GT.-90.) then
            if(mtemp.EQ.1) then
c ---          Kelvin
               tempk=ttest
            elseif(mtemp.EQ.2) then
c ---          Centigrade
               tempk=273.16+ttest
            elseif(mtemp.EQ.3) then
c ---          Farenheit
               tempk=273.16+(ttest-32.0)/1.8
            endif
         endif
      endif

c --- Relative Humidity
      if(icws.GT.0) then
         call IRANGE(icrh,icommas,ncommas,i1,i2)
         call RGET320(char320,i1,i2,rhtest)
         if(rhtest.GE.0.0) then
            rh=frh*rhtest
            irh=NINT(rh)
         endif
      endif

c --- Pressure
      if(icpress.GT.0) then
         call IRANGE(icpress,icommas,ncommas,i1,i2)
         call RGET320(char320,i1,i2,prestest)
         if(prestest.GE.0.0) pres=fpress*prestest
      endif

c --- Cloud observation - amount and height code
      if(icsky.GT.0) then
         call IRANGE(icsky,icommas,ncommas,i1,i2)
         acld='        '
         acld=char320(i1:i2)
         acld=ADJUSTL(acld)
         irain=INDEX(acld,'RA')
         isnow=INDEX(acld,'SN')
         iprecip=irain+isnow
         if(acld(1:3).EQ.'CLR') then
            icc=0
            iceil=999
         elseif(acld(1:3).EQ.'FEW') then
            icc=2
            read(acld(4:6),'(i3)') iceil
            if(iceil.EQ.0) iceil=imiss
         elseif(acld(1:3).EQ.'SCT') then
            icc=4
            read(acld(4:6),'(i3)') iceil
            if(iceil.EQ.0) iceil=imiss
         elseif(acld(1:3).EQ.'BKN') then
            icc=7
            read(acld(4:6),'(i3)') iceil
            if(iceil.EQ.0) iceil=imiss
         elseif(acld(1:3).EQ.'OVC') then
            icc=10
            read(acld(4:6),'(i3)') iceil
            if(iceil.EQ.0) iceil=imiss
         elseif(iprecip.GT.0) then
c ---       Use 1000 feet?
            icc=10
            iceil=10
         endif
      endif

c --- Ceiling
      if(icceil.GT.0) then
         call IRANGE(icceil,icommas,ncommas,i1,i2)
         call RGET320(char320,i1,i2,ceil)
         ceil=fceil*ceil
         iceil=NINT(ceil)
      endif

c --- Cloud Cover
      if(iccld.GT.0) then
         call IRANGE(iccld,icommas,ncommas,i1,i2)
         call RGET320(char320,i1,i2,cld)
         cld=fcld*cld
         icc=NINT(cld)
      endif

c --- Set flag for more data
      idone=0
      return

21    continue
c --- Set flag for end of data
      idone=1

      return
      end

c----------------------------------------------------------------------
      subroutine wrs(iforms,nssta,ispack,io,io6,ibuf,
     &               iyr0,ijul0,ihr0,isec0,iyr,ijul,ihr,isec,
     &               ws,wd,iceil,icc,tempk,irh,pres,ipcode)
c----------------------------------------------------------------------
c
c --- SURFCSV  Version: 7.0.0         Level: 110118                WRS
c ---           J. Scire, D. Strimaitis
c
c --- Write a record of surface meteorological data (one hour of data)
c --- (missing value indicator = 9999. (real) or 9999 (integer))
c --- Option to pack data if output data file is unformatted
c
c --- INPUTS:
c            IFORMS - integer    - Data format flag (1=unformatted,
c                                  2=formatted)
c             NSSTA - integer    - Number of surface stations
c            ISPACK - integer    - Data packing code (0=unpacked,
c                                  1=packed)
c                IO - integer    - Fortran unit no. of surface data
c               IO6 - integer    - Fortran unit no. of list file
c     IBUF(3,NSSTA) - int. array - Buffer to temporarily store packed
c          IYR0,IYR - integer    - Begin,End Year of data (4 digits)
c        IJUL0,IJUL - integer    - Begin,End Julian day of data
c          IHR0,IHR - integer    - Begin,End Hour of data (00-24)
c        ISEC0,ISEC - integer    - Begin,End Seconds of data (0-3600)
c         WS(NSSTA) - real array - Wind speed (m/s)
c         WD(NSSTA) - real array - Wind direction (degrees)
c      ICEIL(NSSTA) - int. array - Ceiling height (hundreds of ft)
c        ICC(NSSTA) - int. array - Opaque sky cover (tenths)
c      TEMPK(NSSTA) - real array - Temperature (deg. K)
c        IRH(NSSTA) - int. array - Relative humidity (percent)
c       PRES(NSSTA) - real array - Surface station pressure (mb)
c     IPCODE(NSSTA) - int. array - Precipitation code
c
c --- OUTPUT:  none
c
c --- WRS called by:  MAIN
c --- WRS calls:      PACKS
c------------------------------------------------------------------------------
c
      real ws(nssta),wd(nssta),tempk(nssta),pres(nssta)
c
      integer iceil(nssta),icc(nssta),irh(nssta),ipcode(nssta)
      integer ibuf(3,nssta)

c --- Keep structure but disable unformatted option here
      if(iforms.eq.1)then
         stop 'Unformatted SURF.DAT is not supported'
c ---    Unformatted option
c ---    code date and time into a single integer variable
         idathr=iyr*100000+ijul*100+ihr
c
         if(ispack.eq.0)then
c
c ---       write unpacked data
            write(io)idathr,ws,wd,iceil,icc,tempk,irh,pres,ipcode
         else
c
c ---       pack and write data
            call packs(nssta,ws,wd,iceil,icc,tempk,irh,pres,ipcode,
     1      ibuf)
            write(io)idathr,ibuf
         endif
      else if(iforms.eq.2)then
c ---    Formatted option
c ---    Snapshot construct
         write(io,10)iyr0,ijul0,ihr0,isec0,iyr,ijul,ihr,isec
10       format(2(3i4,i6.4,2x))
         write(io,20) (ws(n),wd(n),iceil(n),icc(n),tempk(n),irh(n),
     1                 pres(n),ipcode(n),n=1,nssta)
 20      format(1x,f8.3,1x,f8.3,1x,i4,1x,i4,1x,f8.3,1x,i4,1x,f8.3,1x,i4)
      else
         write(io6,30)iforms
30       format(//2x,'ERROR IN SUBR. WRS -- invalid value of IFORMS'/
     1   5x,'IFORMS = ',i10)
         write(*,987)
987      format(1x,'ERROR in SURFCSV run - see SURFCSV.LST file')
         stop
      endif
c
      return
      end
c------------------------------------------------------------------------------
      subroutine packs(nssta,ws,wd,iceil,icc,tempk,irh,pres,ipcode,
     1 ibuf)
c------------------------------------------------------------------------------
c
c --- SMERGE   Version: 4.0       Level: 941215                           PACKS
c ---          J. Scire, SRC
c
c --- Pack a set of surface meteorological variables using integer
c --- packing (3 words/station)
c        Word 1:  TTTTPCRRR --  TTTT = temp. (XXX.X deg, K)
c                                 PC = precipitation code (XX)
c                                RRR = relative humidity (XXX. %)
c        Word 2: pPPPPCCWWW -- pPPPP = station pressure (pXXX.X mb,
c                                      with p = 0 or 1 only)
c                                 CC = opaque sky cover (XX tenths)
c                                WWW = wind direction (XXX. deg.)
c        Word 3:   HHHHSSSS --  HHHH = ceiling height (XXXX. hundreds
c                                      of feet)
c                               SSSS = wind speed (XX.XX m/s)
c
c --- INPUTS:
c             NSSTA - integer    - Number of surface stations
c         WS(NSSTA) - real array - Wind speed (m/s)
c         WD(NSSTA) - real array - Wind direction (degrees)
c      ICEIL(NSSTA) - int. array - Ceiling height (hundreds of ft)
c        ICC(NSSTA) - int. array - Opaque sky cover (tenths)
c      TEMPK(NSSTA) - real array - Temperature (deg. K)
c        IRH(NSSTA) - int. array - Relative humidity (percent)
c       PRES(NSSTA) - real array - Surface station pressure (mb)
c     IPCODE(NSSTA) - int. array - Precipitation code
c
c --- OUTPUT:
c     IBUF(3,NSSTA) - int. array - Array of packed data
c
c --- PACKS called by:  WRS
c --- PACKS calls:      none
c------------------------------------------------------------------------------
      real ws(nssta),wd(nssta),tempk(nssta),pres(nssta)
c
      integer iceil(nssta),icc(nssta),irh(nssta),ipcode(nssta)
      integer ibuf(3,nssta)
c
      do 100 i=1,nssta
c
c --- pack temperature, precip. code, relative humidity into word 1
      if(tempk(i).eq.9999.)then
         it=9999
      else
         it=10.*tempk(i)+0.5
      endif
c
      if(ipcode(i).eq.9999)then
         ipc=99
      else
         ipc=ipcode(i)
      endif
c
      if(irh(i).eq.9999)then
         ir=999
      else
         ir=irh(i)
      endif
      ibuf(1,i)=it*100000+ipc*1000+ir
c
c --- pack station pressure, cloud cover, wind direction into
c --- word 2
      if(pres(i).eq.9999.)then
         ip=9999
      else
c ---    NOTE: 1XXX.X mb is allowed for station pressure and converts to
c ---          1XXXX
         ip=10.*pres(i)+0.5
      endif
c
      if(icc(i).eq.9999)then
         ic=99
      else
         ic=icc(i)
      endif
c
      if(wd(i).eq.9999.)then
         iw=999
      else
         iw=wd(i)+0.5
      endif
      ibuf(2,i)=ip*100000+ic*1000+iw
c
c --- pack ceiling height, wind speed into word 3
      ih=iceil(i)
c
      if(ws(i).eq.9999.)then
         is=9999
      else
         is=100.*ws(i)+0.5
      endif
      ibuf(3,i)=ih*10000+is
100   continue
c
      return
      end
c----------------------------------------------------------------------
      subroutine wrhd(ienvpop,iolst,iform,io,ibyr,ibjul,ibhr,ibsec,
     1                ieyr,iejul,iehr,iesec,atzone,nsta,ipack,idsta,
     2                anem,cname,clat,clon,cver,clev)
c----------------------------------------------------------------------
c
c --- SURFCSV  Version: 7.0.0         Level: 110118               WRHD
c ---          D. Strimaitis
c
c --- Write the header records for a meteorological data file
c
c --- INPUTS:
c           IENVPOP - integer    - Environment toggle for headers
c             IOLST - integer    - Fortran unit no. of list file
c             IFORM - integer    - Data format flag (1=unformatted,
c                                  2=formatted)
c                IO - integer    - Fortran unit no. of data file
c              IBYR - integer    - Beginning year of data (4 digits)
c             IBJUL - integer    - Beginning Julian day number
c              IBHR - integer    - Beginning hour
c             IBSEC - integer    - Beginning second
c              IEYR - integer    - Ending year of data (4 digits)
c             IEJUL - integer    - Ending Julian day number
c              IEHR - integer    - Ending hour
c             IESEC - integer    - Ending second
c            ATZONE - char*8     - Time zone
c                                  UTC-0800 = PST, UTC-0700 = MST, 
c                                  UTC-0600 = CST, UTC-0500 = EST
c              NSTA - integer    - Number of stations
c             IPACK - integer    - Data packing code (0=unpacked,
c                                  1=packed)
c       IDSTA(NSTA) - int. array - Array of station identification
c                                  codes
c     ANEM(NSTA) - real array    - Anemometer height at each station (m)
c    CNAME(NSTA) - char*4 array  - Name of each station (----)
c     CLAT(NSTA) - char*16 array - Latitude of each station (deg[N/S])
c     CLON(NSTA) - char*16 array - Longitude of each station (deg[E/W])
c
c --- OUTPUT:  none
c
c --- WRHD called by:  main
c --- WRHD calls:      none
c----------------------------------------------------------------------
c --- Local Variables
      real anem(nsta)
      integer idsta(nsta)
      character*4 cname(nsta)
      character*16 clat(nsta),clon(nsta)
      character*12 cver,clev
      character*1 q

      character*4 xyunit
      character*8 datum, pmap, atzone
      character*12 daten
      character*16 dataset,dataver
      character*64 datamod
      character*80 comment1

c --- Configure output variables
      data dataset/'SURF.DAT'/, dataver/'2.1'/
      data datamod/'Hour Start and End Times with Seconds'/
      data ncomment/1/
      data comment1/'Produced by SURFCSV Version: '/

c --- Set single quote character
      data q/''''/

c --- Construct the version-level comment string
      j=29
      do i=1,12
         if(cver(i:i).NE.' ') then
            comment1(j:j)=cver(i:i)
            j=j+1
         endif
      enddo
      j=j+1
      comment1(j:j+7)=' Level: '
      j=j+8
      do i=1,12
         if(clev(i:i).NE.' ') then
            comment1(j:j)=clev(i:i)
            j=j+1
         endif
      enddo

c --- Set map projection information
      if(ienvpop.EQ.0) then
         pmap='NONE    '
      elseif(ienvpop.EQ.1) then
         pmap='LL      '
         datum='WGS-G   '
         daten='10-10-2002  '
         xyunit='DEG '
      endif

c --- Write records
      if(iform.eq.1)then

         stop 'Unformatted file (iform.eq.1) not supported'

cc ---    Unformatted file
cc ---    code beginning and ending date/hour
c         ibeg=ibyr*100000+ibjul*100+ibhr
c         iend=ieyr*100000+iejul*100+iehr
c         write(io) dataset,dataver,datamod
c
c         write(io) ncomment
c         do ic=1,ncomment
c            write(io) comment1
c         enddo
c
c         write(io) pmap
c         if(ienvpop.EQ.0) then
c            write(io)ibeg,iend,ibtz,nsta,ipack
c            write(io)idsta
c         elseif(ienvpop.EQ.1) then
c            write(io) datum,daten
c            write(io) xyunit
c            write(io)ibeg,iend,ibtz,nsta,ipack
c            write(io)idsta
c            write(io)cname
c            write(io)clat
c            write(io)clon
c            write(io)anem
c         endif

      else if(iform.eq.2)then
c ---    Formatted file
         write(io,'(2a16,a64)') dataset,dataver,datamod

         write(io,'(i4)') ncomment
         do ic=1,ncomment
            write(io,'(a80)') comment1
         enddo

         write(io,'(a8)') pmap
         if(ienvpop.EQ.0) then
            write(io,'(a8)')atzone
            write(io,10)ibyr,ibjul,ibhr,ibsec,
     &                  ieyr,iejul,iehr,iesec,nsta
10          format(2(i6,2i4,i6.4),i5)
            do i=1,nsta
               write(io,20)idsta(i)
            enddo
20          format(i8)
         elseif(ienvpop.EQ.1) then
            write(io,'(a8,a12)') datum,daten
            write(io,'(a4)') xyunit
            write(io,'(a8)')atzone
            write(io,10)ibyr,ibjul,ibhr,ibsec,
     &                  ieyr,iejul,iehr,iesec,nsta
            do i=1,nsta
               write(io,21)idsta(i),q,cname(i),q,q,clat(i),q,q,
     &                     clon(i),q,anem(i)
            enddo
21          format(i8,2x,a1,a4,a1,2x,2(a1,a16,a1,2x),f10.2)
         endif
      else
         write(iolst,12)iform
12       format(//2x,'ERROR IN SUBR. WRHD -- invalid value of IFORM'/
     1   5x,'IFORM = ',i10)
         write(*,987)
987      format(1x,'ERROR in SURFCSV run - see SURFCSV.LST file')
         stop
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine yr4(io,iyr,ierr)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.571    Level: 991104                    YR4
c ---            D. Strimaitis,   Earth Tech
c
c --- PURPOSE: Checks/converts 2-digit year to 4-digit year
c
c --- INPUTS:
c                IO - integer    - Unit number for list file output
c               IYR - integer    - Year (YYYY or YY)
c
c     Common block /Y2K/:
c             IYYLO - integer    - Smallest 2-digit year for which
c                                  'old' century marker is used
c             ICCLO - integer    - 2-digit ('old') century
c
c --- OUTPUT:
c               IYR - integer    - Year (YYYY)
c              IERR - integer    - Error code: 0=OK, 1=FATAL
c
c --- YR4 called by:  Input routines reading 'year' data
c --- YR4 calls:      none
c----------------------------------------------------------------------
c
      common/y2k/iyylo,icclo

      ierr=0

c --- Test for 4-digit year (must exceed 1000)
      if(iyr.GT.1000) then
c ---    Passes 11th Century test (large year not trapped)
         return
      elseif(iyr.LT.100 .AND. iyr.GE.0) then
c ---    2-digit year
c ---    Construct 4-digit year
         if(iyr.LT.iyylo) then
            iyr=(icclo+1)*100+iyr
         else
            iyr=icclo*100+iyr
         endif
      else
c ---    Year not recognized
         ierr=1
         write(io,*)'ERROR in YR4 --- Year not recognized: ',iyr
         write(*,*)'ERROR in YR4 --- Year not recognized: ',iyr
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine qayr4(io,iyr,metrun,ierr)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.571    Level: 991104                  QAYR4
c ---            D. Strimaitis,   Earth Tech
c
c --- PURPOSE: Defines century and year markers to use in converting
c ---          2-digit year to 4-digit year
c ---          The IBYR (YYYY) must be provided in the control file
c
c --- INPUTS:
c                IO - integer    - Unit number for list file output
c               IYR - integer    - Year provided for start of run
c            METRUN - integer    - Flag to run period in met file
c                                  0 = do not run period
c                                  1 = run period
c
c --- OUTPUT:
c              IERR - integer    - Error code: 0=OK, 1=FATAL
c
c     Common block /Y2K/:
c             IYYLO - integer    - Smallest 2-digit year for which
c                                  'old' century marker is used
c             ICCLO - integer    - 2-digit ('old') century
c
c --- QAYR4 called by:  host subroutines
c --- QAYR4 calls:      none
c----------------------------------------------------------------------
c
      common/y2k/iyylo,icclo

c --- Sets parameters for the starting century marker (CC) and the
c --- 2-digit year (YY) used as the marker between the starting century
c --- and the next century.  For example, if CC=19 and YY=30, then a
c --- year less than 30 (say 15) is assumed to be 2015.  Any year
c --- greater than or equal to 30 (say 56) is assumed to be 1956.

c --- Set number of years prior to start of simulation that must not
c --- be placed in the next century
      data ibackyr/50/

      ierr=0

c --- Expect explicit starting year (YYYY)
c --- Test for 4-digit year (must exceed 1000)
      if(iyr.GT.1000) then
c ---    Passes 11th Century test (large year not trapped)
c ---    Back up IBACKYR years to set IYYLO
         kyr=iyr-ibackyr
c ---    Extract starting 2-digit century and 2-digit year
         icclo=kyr/100
         iyylo=kyr-icclo*100

c ---    Warn user that control file input is used to convert to YYYY
         iyr1=icclo*100+iyylo
         iyr2=(icclo+1)*100+iyylo-1
         write(io,*)
         write(io,*)'-------------------------------------------------'
         write(io,*)'NOTICE: Starting year in control file sets the'
         write(io,*)'        expected century for the simulation.  All'
         write(io,*)'        YY years are converted to YYYY years in'
         write(io,*)'        the range: ',iyr1,iyr2
         write(io,*)'-------------------------------------------------'
         write(io,*)
      else
         ierr=1
         write(*,*)
         write(*,*)'--------------------------------------------'
         write(*,*)'QAYR4 -- Start year must be 4-digits!: ',iyr
         if(metrun.EQ.1) then
            write(*,*)'         and must always be provided'
         endif
         write(*,*)'--------------------------------------------'
         write(*,*)
         write(io,*)
         write(io,*)'-------------------------------------------'
         write(io,*)'QAYR4 -- Start year must be 4-digits!: ',iyr
         if(metrun.EQ.1) then
            write(io,*)'         and must always be provided'
         endif
         write(io,*)'-------------------------------------------'
         write(io,*)
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine julday(io,iyr,imo,iday,ijuldy)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.571    Level: 000602                 JULDAY
c ---            J. Scire, SRC
c
c --- PURPOSE:  Compute the Julian day number from the Gregorian
c               date (month, day)
c
c --- UPDATE
c ---               000602  (DGS): YYYY format for year
c
c --- INPUTS:
c            IO - integer      - Unit number for list file output
c           IYR - integer      - Year
c           IMO - integer      - Month
c          IDAY - integer      - Day
c
c --- OUTPUT:
c          IJUL - integer      - Julian day
c
c --- JULDAY called by:  host subroutines
c --- JULDAY calls:      none
c----------------------------------------------------------------------
c
      integer kday(12)
      data kday/0,31,59,90,120,151,181,212,243,273,304,334/
c
c --- Check for valid input data
      ierr=0
c --- Check for valid month
      if(imo.lt.1.or.imo.gt.12)ierr=1
c --- Check for valid day in 30-day months
      if(imo.eq.4.or.imo.eq.6.or.imo.eq.9.or.imo.eq.11)then
         if(iday.gt.30)ierr=1
      else if(imo.eq.2)then
         if(mod(iyr,4).eq.0)then
c ---       February in a leap year
            if(iday.gt.29)ierr=1
         else
c ---       February in a non-leap year
            if(iday.gt.28)ierr=1
         endif
      else
c ---    Check for valid day in 31-day months
         if(iday.gt.31)ierr=1
      endif
c
      if(ierr.eq.1)then
         write(io,*)
         write(io,*)'ERROR in SUBR. JULDAY'
         write(io,*)'Invalid date - IYR = ',iyr,' IMO = ',
     1    imo,' IDAY = ',iday
         write(*,*)
         stop 'Halted in JULDAY -- see list file.'
      endif
c
c --- Compute the Julian day
      ijuldy=kday(imo)+iday
      if(imo.le.2)return
      if(mod(iyr,4).EQ.0)ijuldy=ijuldy+1
c
      return
      end
c------------------------------------------------------------------------------
      subroutine dedat(idathr,iyr,ijul,ihr)
c------------------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.571    Level: 941215                  DEDAT
c ---            J. Scire, SRC
c
c --- Decode a date-time variable
c
c --- INPUTS:
c            IDATHR - integer    - Date-time variable (YYYYJJJHH)
c
c --- OUTPUT:
c               IYR - integer    - Year of precip. data (4 digits)
c              IJUL - integer    - Julian day number of precip. data
c               IHR - integer    - Ending hour (1-24) of precip. data
c
c --- DEDAT called by:  host subroutines
c --- DEDAT calls:      none
c------------------------------------------------------------------------------
c
c --- decode date and time
      iyr=idathr/100000
      ijul=idathr/100-iyr*1000
      ihr=idathr-iyr*100000-ijul*100
c
      return
      end
c----------------------------------------------------------------------
      subroutine incrs(io,iyr,ijul,ihr,isec,nsec)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.57     Level: 061020                  INCRS
c                D. Strimaitis, EARTH TECH
c
c --- PURPOSE:  Increment the time and date by "NSEC" seconds
c
c --- UPDATE
c --- V2.54 (061020) from V2.4 (041029) (DGS)
c               - Allow negative increment
c
c --- INPUTS:
c       IO     - integer - Unit number for list file output
c       IYR    - integer - Current year (YYYY)
c       IJUL   - integer - Current Julian day (JJJ)
c       IHR    - integer - Current hour (00-23)
c       ISEC   - integer - Current second (0000-3599)
c       NSEC   - integer - Time increment (seconds)
c       Parameters: IO6
c
c --- OUTPUT:
c       IYR    - integer - Updated year
c       IJUL   - integer - Updated Julian day
c       IHR    - integer - Updated hour (00-23)
c       ISEC   - integer - Updated seconds (0000-3599)
c
c --- INCRS called by: host subroutines
c --- INCRS calls:     INCR
c----------------------------------------------------------------------

      if(nsec.GE.0) then
c ---    Increment seconds
         isec=isec+nsec
         if(isec.GE.3600) then
            nhrinc=isec/3600
            isec=MOD(isec,3600)
            call INCR(io,iyr,ijul,ihr,nhrinc)
         endif

      else
c ---   Decrement seconds
         isec=isec+nsec
         if(isec.LT.0) then
c ---       Earlier hour
            ksec=-isec
            if(ksec.GE.3600) then
c ---          Back up at least 1 hour
               nhrinc=ksec/3600
               ksec=MOD(ksec,3600)
               nhrinc=-nhrinc
               call INCR(io,iyr,ijul,ihr,nhrinc)
            endif
            isec=-ksec
            if(isec.LT.0) then
c ---          Back up 1 more hour
               nhrinc=-1
               isec=3600+isec
               call INCR(io,iyr,ijul,ihr,nhrinc)
            endif
         endif

      endif

      return
      end
c----------------------------------------------------------------------
      subroutine incr(io,iyr,ijul,ihr,nhrinc)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.571    Level: 000602                   INCR
c                J. Scire, SRC
c
c --- PURPOSE:  Increment the time and date by "NHRINC" hours
c
c --- UPDATE
c ---               000602  (DGS): add message to "stop"
c ---               980304  (DGS): Allow for a negative "increment" of
c                                  up to 24 hours
c ---               980304  (DGS): Allow for arbitrarily large nhrinc
c
c --- INPUTS:
c       IO     - integer - Unit number for list file output
c       IYR    - integer - Current year
c       IJUL   - integer - Current Julian day
c       IHR    - integer - Current hour (00-23)
c       NHRINC - integer - Time increment (hours)
c
c               NOTE: "NHRINC" must >= -24
c                      Hour is between 00-23
c
c --- OUTPUT:
c       IYR    - integer - Updated year
c       IJUL   - integer - Updated Julian day
c       IHR    - integer - Updated hour (00-23)
c
c --- INCR called by: host subroutines
c --- INCR calls:     none
c----------------------------------------------------------------------
c
c --- Check nhrinc
      if(nhrinc.lt.-24) then
         write(io,*)'ERROR IN SUBR. INCR -- Invalid value of NHRINC ',
     1   '-- NHRINC = ',nhrinc
         write(*,*)
         stop 'Halted in INCR -- see list file.'
      endif

c --- Save increment remaining (needed if nhrinc > 8760)
      nleft=nhrinc
c
c --- Process change in hour
      if(nhrinc.gt.0)then
c
10       ninc=MIN0(nleft,8760)
         nleft=nleft-ninc
c
c ---    Increment time
         ihr=ihr+ninc
         if(ihr.le.23)return
c
c ---    Increment day
         ijul=ijul+ihr/24
         ihr=mod(ihr,24)
c
c ---    ILEAP = 0 (non-leap year) or 1 (leap year)
         if(mod(iyr,4).eq.0)then
            ileap=1
         else
            ileap=0
         endif
c
         if(ijul.gt.365+ileap) then
c ---       Update year
            iyr=iyr+1
            ijul=ijul-(365+ileap)
         endif
c
c ---    Repeat if more hours need to be added
         if(nleft.GT.0) goto 10
c
      elseif(nhrinc.lt.0)then
c ---    Decrement time
         ihr=ihr+nhrinc
         if(ihr.lt.0)then
            ihr=ihr+24
            ijul=ijul-1
            if(ijul.lt.1)then
               iyr=iyr-1
               if(mod(iyr,4).eq.0)then
                  ijul=366
               else
                  ijul=365
               endif
            endif
         endif
      endif
c
      return
      end


c----------------------------------------------------------------------
      subroutine COLUMN320(c320,nlim,cd,ncol,nc,ic)
c----------------------------------------------------------------------
c
c --- METSERIES Version: 1.79      Level: 081017              COLUMN320
c ---           D. Strimaitis
c
c --- PURPOSE:  Identifies location of all column delimiters in a
c               string and checks for expected number
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c               CD - character*1       - Column delimiter character
c             NCOL - integer           - Number of columns expected
c                                        (Enter 99 if unknown)
c
c --- OUTPUT:
c
c             NCOL - integer           - Number of columns processed
c               NC - integer           - Number of delimiters
c       IC(0:NCOL) - integer           - Location of delimiters
c
c --- COLUMN320 called by: (utility)
c --- COLUMN320 calls:      none
c----------------------------------------------------------------------

c      include 'params.ser'

      character*1 cd
      character*320 c320
      integer ic(0:ncol)
      logical lncol

      if(nlim.GT.320) then
c         write(ilog,*)'ERROR in Subr. COLUMN320 - string is too long'
c         write(ilog,*)'Maximum length  : 320'
c         write(ilog,*)'Requested length: ',nlim
         write(*,*)'ERROR in Subr. COLUMN320 - string is too long'
         write(*,*)'Maximum length  : 320'
         write(*,*)'Requested length: ',nlim
         stop 'Program HALTED'
      endif

c --- Set logical for processing a specific number of columns
      if(ncol.GT.99) then
c         write(ilog,*)'ERROR in Subr. COLUMN320 - too many columns'
c         write(ilog,*)'Maximum number  : 99'
c         write(ilog,*)'Number requested: ',ncol
         write(*,*)'ERROR in Subr. COLUMN320 - too many columns'
         write(*,*)'Maximum number  : 99'
         write(*,*)'Number requested: ',ncol
         stop 'Program HALTED'
      endif
      lncol=.FALSE.
      if(ncol.LT.99) lncol=.TRUE.

      nc=0
      ic(0)=0
      do k=1,nlim
         if(c320(k:k).EQ.cd) then
            if(LNCOL) then
c ---          Actual number known
               if(nc.LT.ncol) then
                  nc=nc+1
                  ic(nc)=k
               endif
            else
c ---          Actual number unknown
               nc=nc+1
               ic(nc)=k
               ncol=nc
            endif
         endif
      enddo
c --- Last delimiter in line is usually missing
      if(LNCOL) then
         if(nc.LT.ncol) then
            nc=nc+1
            ic(nc)=nlim+1
         endif
      else
         nc=nc+1
         ic(nc)=nlim+1
         ncol=nc
      endif
c --- Check number of columns
      if(nc.LT.ncol) then
c         write(ilog,*)'ERROR in Subr. COLUMN320 - '
c         write(ilog,*)'Too few columns in the line'
c         write(ilog,*)'Expected number of columns: ',ncol
c         write(ilog,*)'                     Found: ',nc
c         write(ilog,'(a)')c320(1:nlim)
         write(*,*)'ERROR in Subr. COLUMN320 - '
         write(*,*)'Too few columns in the line'
         write(*,*)'Expected number of columns: ',ncol
         write(*,*)'                     Found: ',nc
         write(*,'(a)')c320(1:nlim)
         stop 'Program HALTED'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine IRANGE(n,ic,nc,i1,i2)
c----------------------------------------------------------------------
c
c --- METSERIES Version: 1.79      Level: 081017                 IRANGE
c ---           D. Strimaitis
c
c --- PURPOSE:  Extracts starting and ending position of field for a
c               particular column in a CSV line
c
c --- INPUTS:
c
c                N - integer           - Column number for field
c         IC(0:NC) - integer           - Location of commas
c               NC - integer           - Number of commas
c
c --- OUTPUT:
c
c               I1 - integer           - Starting position of field
c               I2 - integer           - Ending position of field
c
c --- IRANGE called by: (utility)
c --- IRANGE calls:      none
c----------------------------------------------------------------------

      integer ic(0:nc)

      i1=ic(n-1)+1
      i2=ic(n)-1

      return
      end

c----------------------------------------------------------------------
      subroutine rget320(c320,i1,i2,x)
c----------------------------------------------------------------------
c
c --- METSERIES Version: 1.79      Level: 081017                RGET320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert character string to a real.  This sub is needed
c               when there is no decimal point.  The LF95 compiler
c               does not do the internal read conversion without the '.'
c               Also, determine when exponential notation is present.
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c               i1 - integer           - First character of substring
c               i2 - integer           - Last character of substring
c
c --- OUTPUT:
c
c                X - real              - Output real
c
c --- RGET320 called by: (utility)
c --- RGET320 calls:      none
c----------------------------------------------------------------------
      character*320 c320
      character*20 c20, blnk20
      data blnk20/'                    '/

      if(i1.GT.i2) return

      c20=blnk20
      c20=c320(i1:i2)

c --- Left justify field
      c20=ADJUSTL(c20)

c --- Test for exponential notation
      ie=MAX(INDEX(c20,'e'),INDEX(c20,'E'))

      if(ie.EQ.0) then
c ---    Simple real number
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            ipt=LEN_TRIM(c20)+1
            c20(ipt:ipt)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(f)') x

      else
c ---    Exponential notation
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            do k=20,ie+1,-1
               km1=k-1
               c20(k:k)=c20(km1:km1)
            enddo
            c20(ie:ie)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(e)') x

      endif

      return
      end
c----------------------------------------------------------------------
      subroutine case320(mode,c320,nlim)
c----------------------------------------------------------------------
c
c --- PTVOL     Version: 1.32     Level: 060814                CASE320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert all letters within a character string to 
c               either upper or lower case
c
c --- INPUTS:
c
c             MODE - character*5       - Output directive for either 
c                                        upper or lower case
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c                                        uniform case
c
c --- CASE320 called by: (utility)
c --- CASE320 calls:      IACHAR, CHAR
c----------------------------------------------------------------------

      character*320 c320
      character*5 mode

      if(nlim.GT.320) then
         write(*,*)'ERROR in Subr. CASE320 - string is too long'
         write(*,*)'Maximum length  : 320'
         write(*,*)'Requested length: ',nlim
         stop 'Program HALTED'
      endif

      if(mode.EQ.'UPPER' .OR. mode.EQ.'upper') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.97 .AND. ia.LE.122) then
               ia=ia-32
               c320(k:k)=CHAR(ia)
            endif
         enddo

      elseif(mode.EQ.'LOWER' .OR. mode.EQ.'lower') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.65 .AND. ia.LE.90) then
               ia=ia+32
               c320(k:k)=CHAR(ia)
            endif
         enddo
      else
         write(*,*)'ERROR in Subr. CASE320 - invalid output mode'
         write(*,*)'Expected UPPER or LOWER'
         write(*,*)'Found: ',mode
         stop 'Program HALTED'
      endif

      return
      end
